//
//  ViewController.swift
//  DemoPay
//
//  Created by Flex on 11/21/16.
//  Copyright © 2016 Flex. All rights reserved.
//

import UIKit


class SettingsViewController: UIViewController, UITableViewDelegate, UITableViewDataSource, DTDeviceDelegate{

    @IBOutlet var tvInfo: UITextView!
    @IBOutlet var tvSettings: UITableView!

    var btleDevices = [CBPeripheral]()

    let settings=[
        "Discover Devices"
        ]

    let lib=DTDevices.sharedDevice() as! DTDevices

    func bluetoothLEDeviceConnected(_ device: CBPeripheral!) {
        tvSettings.reloadData()
    }

    func bluetoothLEDeviceDisconnected(_ device: CBPeripheral!) {
        tvSettings.reloadData()
    }

    func connectionState(_ state: Int32) {
        var info="SDK: ver \(lib.sdkVersion/100).\(String.init(format: "%02d", lib.sdkVersion%100)) \(DateFormatter.localizedString(from: lib.sdkBuildDate, dateStyle: DateFormatter.Style.medium, timeStyle: DateFormatter.Style.none))\n"

        do
        {
            if state==CONN_STATES.CONNECTED.rawValue
            {
                let connected = try lib.getConnectedDevicesInfo()
                for device in connected
                {
                    info+="\(device.name!) \(device.model!) connected\nFW Rev: \(device.firmwareRevision!) HW Rev: \(device.hardwareRevision!)\nSerial: \(device.serialNumber!)\n"
                }
            }else {
            }
        }catch {}
        tvInfo.text=info
    }

    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        return "Bluetooth LE Devices"
    }

    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return btleDevices.count+1
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell=tvSettings.dequeueReusableCell(withIdentifier: "SettingsCell", for: indexPath)

        if indexPath.row == 0 {
            cell.textLabel?.text=settings[indexPath.row]
            cell.detailTextLabel?.text=""
        }else {
            let device=btleDevices[indexPath.row-1]

            cell.textLabel?.text=device.name
            cell.detailTextLabel?.text=device.identifier.uuidString

            if lib.btleConnectedDevices != nil && lib.btleConnectedDevices.contains(device) {
                cell.accessoryType = .checkmark
            }else {
                cell.accessoryType = .none
            }
        }

        return cell
    }

    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        defer {
            Progress.hide()
            tvSettings.reloadData()
        }

        do {
            if indexPath.row==0
            {//perform discovery
                try btleDevices = lib.btleDiscoverSupportedDevices(BLUETOOTH_FILTER.ALL.rawValue, stopOnFound: false)
            }else
            {//connect to the device
                Progress.show(self)

                let selectedDevice=btleDevices[indexPath.row-1]

                if lib.btleConnectedDevices != nil && lib.btleConnectedDevices.contains(selectedDevice)
                {
                    try lib.btleDisconnect(selectedDevice);
                    ViewController.instance.fiscalDisconnect()
                }else
                {
//                    try lib.btleConnect(toDeviceAndGetStreams: selectedDevice)
//                    ViewController.instance.fiscalConnect(inStream: lib.btleInputStream, outStream: lib.btleOutputStream)
                    try lib.btleConnect(toDevice: selectedDevice)
                }
            }
        } catch let error as NSError {
            Progress.hide()
            self.showError("Bluetooth Error", error: error)
        }
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        lib.addDelegate(self)
        tvSettings.reloadData()
    }

    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)

        lib.removeDelegate(self)
    }


    override func viewDidLoad() {
        super.viewDidLoad()

        tvSettings.delegate=self
        tvSettings.dataSource=self

        tvSettings.backgroundColor = .clear
        tvSettings.tableFooterView = UIView()

        self.connectionState(lib.connstate)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }


}


